@@ -42,11 +42,6 @@
 #include "gtksecentry.h"
 #include "memory.h"
 
-#ifndef _
-#  include <libintl.h>
-#  define _(x) gettext(x)
-#endif
-
 #define MIN_SECURE_ENTRY_WIDTH  150
 #define DRAW_TIMEOUT     20
 #define INNER_BORDER     2
@@ -270,7 +265,7 @@ gboolean g_use_secure_mem = FALSE;
 
 
 gpointer
-g_malloc(gulong size)
+secentry_malloc (GMALLOC_SIZE size)
 {
     gpointer p;
 
@@ -287,28 +282,9 @@ g_malloc(gulong size)
     return p;
 }
 
-gpointer
-g_malloc0(gulong size)
-{
-    gpointer p;
-
-    if (size == 0)
-	return NULL;
-
-    if (g_use_secure_mem) {
-	p = (gpointer) secmem_malloc(size);
-	if (p)
-	    memset(p, 0, size);
-    } else
-	p = (gpointer) calloc(size, 1);
-    if (!p)
-	g_error("could not allocate %ld bytes", size);
-
-    return p;
-}
 
 gpointer
-g_realloc(gpointer mem, gulong size)
+secentry_realloc (gpointer mem, GMALLOC_SIZE size)
 {
     gpointer p;
 
@@ -338,7 +314,7 @@ g_realloc(gpointer mem, gulong size)
 }
 
 void
-g_free(gpointer mem)
+secentry_free(gpointer mem)
 {
     if (mem) {
 	if (m_is_secure(mem))
@@ -452,27 +428,24 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
     g_object_class_install_property(gobject_class,
 				    PROP_CURSOR_POSITION,
 				    g_param_spec_int("cursor_position",
-						     _("Cursor Position"),
-						     _
-						     ("The current position of the insertion cursor in chars"),
+						     "Cursor Position",
+						     "The current position of the insertion cursor in chars",
 						     0, MAX_SIZE, 0,
 						     G_PARAM_READABLE));
 
     g_object_class_install_property(gobject_class,
 				    PROP_SELECTION_BOUND,
 				    g_param_spec_int("selection_bound",
-						     _("Selection Bound"),
-						     _
-						     ("The position of the opposite end of the selection from the cursor in chars"),
+						     "Selection Bound",
+						     "The position of the opposite end of the selection from the cursor in chars",
 						     0, MAX_SIZE, 0,
 						     G_PARAM_READABLE));
 
     g_object_class_install_property(gobject_class,
 				    PROP_MAX_LENGTH,
 				    g_param_spec_int("max_length",
-						     _("Maximum length"),
-						     _
-						     ("Maximum number of characters for this entry. Zero if no maximum"),
+						     "Maximum length",
+						     "Maximum number of characters for this entry. Zero if no maximum",
 						     0, MAX_SIZE, 0,
 						     G_PARAM_READABLE |
 						     G_PARAM_WRITABLE));
@@ -480,9 +453,8 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
     g_object_class_install_property(gobject_class,
 				    PROP_HAS_FRAME,
 				    g_param_spec_boolean("has_frame",
-							 _("Has Frame"),
-							 _
-							 ("FALSE removes outside bevel from entry"),
+							 "Has Frame",
+							 "FALSE removes outside bevel from entry",
 							 TRUE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -490,10 +462,8 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
     g_object_class_install_property(gobject_class,
 				    PROP_INVISIBLE_CHAR,
 				    g_param_spec_unichar("invisible_char",
-							 _
-							 ("Invisible character"),
-							 _
-							 ("The character to use when masking entry contents (in \"password mode\")"),
+							 "Invisible character",
+							 "The character to use when masking entry contents (in \"password mode\")",
 							 '*',
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -502,16 +472,14 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
 				    PROP_ACTIVATES_DEFAULT,
 				    g_param_spec_boolean
 				    ("activates_default",
-				     _("Activates default"),
-				     _
-				     ("Whether to activate the default widget (such as the default button in a dialog) when Enter is pressed"),
+				     "Activates default",
+				     "Whether to activate the default widget (such as the default button in a dialog) when Enter is pressed",
 				     FALSE,
 				     G_PARAM_READABLE | G_PARAM_WRITABLE));
     g_object_class_install_property(gobject_class, PROP_WIDTH_CHARS,
 				    g_param_spec_int("width_chars",
-						     _("Width in chars"),
-						     _
-						     ("Number of characters to leave space for in the entry"),
+						     "Width in chars",
+						     "Number of characters to leave space for in the entry",
 						     -1, G_MAXINT, -1,
 						     G_PARAM_READABLE |
 						     G_PARAM_WRITABLE));
@@ -519,18 +487,16 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
     g_object_class_install_property(gobject_class,
 				    PROP_SCROLL_OFFSET,
 				    g_param_spec_int("scroll_offset",
-						     _("Scroll offset"),
-						     _
-						     ("Number of pixels of the entry scrolled off the screen to the left"),
+						     "Scroll offset",
+						     "Number of pixels of the entry scrolled off the screen to the left",
 						     0, G_MAXINT, 0,
 						     G_PARAM_READABLE));
 
     g_object_class_install_property(gobject_class,
 				    PROP_TEXT,
 				    g_param_spec_string("text",
-							_("Text"),
-							_
-							("The contents of the entry"),
+							"Text",
+							"The contents of the entry",
 							"",
 							G_PARAM_READABLE |
 							G_PARAM_WRITABLE));
@@ -684,13 +650,6 @@ gtk_secure_entry_class_init(GtkSecureEntryClass * class)
 				 GDK_CONTROL_MASK, "delete_from_cursor", 2,
 				 G_TYPE_ENUM, GTK_DELETE_WORD_ENDS,
 				 G_TYPE_INT, -1);
-
-    gtk_settings_install_property(g_param_spec_boolean
-				  ("gtk-entry-select-on-focus",
-				   _("Select on focus"),
-				   _
-				   ("Whether to select the contents of an entry when it is focused"),
-				   TRUE, G_PARAM_READWRITE));
 }
 
 static void
@@ -1368,14 +1327,18 @@ gtk_secure_entry_focus_out(GtkWidget * widget, GdkEventFocus * event)
 static void
 gtk_secure_entry_grab_focus(GtkWidget * widget)
 {
+    GtkWidget *tmp;
     GtkSecureEntry *entry = GTK_SECURE_ENTRY(widget);
     gboolean select_on_focus;
 
     GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
     GTK_WIDGET_CLASS(parent_class)->grab_focus(widget);
 
-    g_object_get(gtk_widget_get_settings(widget),
+    /* read current select on focus setting from GtkEntry */
+    tmp = gtk_entry_new ();
+    g_object_get(gtk_widget_get_settings(tmp),
 		 "gtk-entry-select-on-focus", &select_on_focus, NULL);
+    gtk_widget_destroy (tmp);
 
     if (select_on_focus && !entry->in_click)
 	gtk_editable_select_region(GTK_EDITABLE(widget), 0, -1);
@@ -3210,11 +3173,11 @@ static gint
 get_cursor_time(GtkSecureEntry * entry)
 {
     GtkSettings *settings = gtk_widget_get_settings(GTK_WIDGET(entry));
-    gint time;
+    gint blinktime;
 
-    g_object_get(settings, "gtk-cursor-blink-time", &time, NULL);
+    g_object_get(settings, "gtk-cursor-blink-time", &blinktime, NULL);
 
-    return time;
+    return blinktime;
 }
 
 static void
