@@ -1,5 +1,5 @@
 /* pinentry.h - The interface for the PIN entry support library.
-   Copyright (C) 2002, 2003 g10 Code GmbH
+   Copyright (C) 2002, 2003, 2010 g10 Code GmbH
    
    This file is part of PINENTRY.
    
@@ -14,9 +14,8 @@
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA  */
+   along with this program; if not, see <http://www.gnu.org/licenses/>. 
+ */
 
 #ifndef PINENTRY_H
 #define PINENTRY_H
@@ -28,9 +27,20 @@ extern "C" {
 #endif
 #endif
 
+#undef ENABLE_ENHANCED
+
+typedef enum {
+  PINENTRY_COLOR_NONE, PINENTRY_COLOR_DEFAULT,
+  PINENTRY_COLOR_BLACK, PINENTRY_COLOR_RED,
+  PINENTRY_COLOR_GREEN, PINENTRY_COLOR_YELLOW,
+  PINENTRY_COLOR_BLUE, PINENTRY_COLOR_MAGENTA,
+  PINENTRY_COLOR_CYAN, PINENTRY_COLOR_WHITE
+} pinentry_color_t;
 
 struct pinentry
 {
+  /* The window title, or NULL.  */
+  char *title;
   /* The description to display, or NULL.  */
   char *description;
   /* The error message to display, or NULL.  */
@@ -39,6 +49,8 @@ struct pinentry
   char *prompt;
   /* The OK button text to display, or NULL.  */
   char *ok;
+  /* The Not-OK button text to display, or NULL.  */
+  char *notok;
   /* The Cancel button text to display, or NULL.  */
   char *cancel;
   /* The buffer to store the secret into.  */
@@ -59,22 +71,65 @@ struct pinentry
 
   /* True if debug mode is requested.  */
   int debug;
+
+#ifdef ENABLE_ENHANCED
   /* True if enhanced mode is requested.  */
   int enhanced;
+#endif
+
   /* True if caller should grab the keyboard.  */
   int grab;
   /* The window ID of the parent window over which the pinentry window
      should be displayed.  */
   int parent_wid;
 
+  /* The name of an optional file which will be touched after a curses
+     entry has been displayed.  */
+  char *touch_file;
+
   /* The user should set this to -1 if the user canceled the request,
      and to the length of the PIN stored in pin otherwise.  */
   int result;
 
+  /* The user should set this if the NOTOK button was pressed.  */
+  int canceled;
+
   /* The user should set this to true if an error with the local
      conversion occured. */
   int locale_err;
 
+  /* The caller should set this to true if only one button is
+     required.  This is useful for notification dialogs where only a
+     dismiss button is required. */
+  int one_button;
+
+  /* If this is not NULL, a passphrase quality indicator is shown.
+     There will also be an inquiry back to the caller to get an
+     indication of the quality for the passphrase entered so far.  The
+     string is used as a label for the quality bar.  */
+  char *quality_bar;
+
+  /* The tooltip to be show for the qualitybar.  Malloced or NULL.  */
+  char *quality_bar_tt;
+
+  /* For the curses pinentry, the color of error messages.  */
+  pinentry_color_t color_fg;
+  int color_fg_bright;
+  pinentry_color_t color_bg;
+  pinentry_color_t color_so;
+  int color_so_bright;
+
+  /* Malloced and i18ned default strings or NULL.  These strings may
+     include an underscore character to indicate an accelerator key.
+     A double underscore represents a plain one.  */
+  char *default_ok;
+  char *default_cancel;
+  char *default_prompt;
+
+  /* For the quality indicator we need to do an inquiry.  Thus we need
+     to save the assuan ctx.  */
+  void *ctx_assuan;
+
 };
 typedef struct pinentry *pinentry_t;
 
@@ -102,6 +157,11 @@ char *pinentry_utf8_to_local (char *lc_ctype, char *text);
    Return NULL on error. */
 char *pinentry_local_to_utf8 (char *lc_ctype, char *text, int secure);
 
+
+/* Run a quality inquiry for PASSPHRASE of LENGTH. */
+int pinentry_inq_quality (pinentry_t pin, 
+                          const char *passphrase, size_t length);
+
 /* Try to make room for at least LEN bytes for the pin in the pinentry
    PIN.  Returns new buffer on success and 0 on failure.  */
 char *pinentry_setbufferlen (pinentry_t pin, int len);
